#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['fake_user_agent']

package_data = \
{'': ['*'], 'fake_user_agent': ['ideas/*']}

install_requires = \
['lxml', 'aiohttp']

entry_points = \
{'console_scripts': ['fakeua = fake_user_agent.user_agent:get_input']}

setup(name='fake_user_agent',
      version='2.1.4',
      description='Randomly generates a useragent for fetching a web page without a browser.',
      author='Kate Wang',
      author_email='kate.wang2018@gmail.com',
      url='https://github.com/KateWang2016/fake_user_agent',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
